@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: engine.h,v 1.4 2004/11/05 23:11:46 dsalt Exp $
+ * $Id: engine.h,v 1.4.6.1 2005/04/29 20:01:44 dsalt Exp $
  *
  * init xine engine, set up script engine with player object
  */
@@ -32,7 +32,7 @@ int engine_exec (const char *cmd, se_print_cb_t cb, void *cb_data);
 int v_engine_exec (const char *cmd, se_print_cb_t cb, void *cb_data, ...);
 
 /* non-zero on failure */
-#define engine_mutex_trylock(s) \
+#define engine_mutex_trylock(se) \
   (!(!pthread_mutex_trylock (&engine_lock) || \
      (se->print_cb (se->print_cb_data, "sorry - engine busy."), 0)))
 
