@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gtkvideo.c,v 1.41.2.3 2005/06/28 18:07:26 dsalt Exp $
+ * $Id: gtkvideo.c,v 1.41.2.4 2005/09/12 00:47:38 dsalt Exp $
  *
  * gtk xine video widget
  *
@@ -1549,6 +1549,37 @@ gboolean gtk_video_get_use_post_plugins_audio (GtkVideo *gtv)
 
 /* Plugins: audio visualisation post-plugins */
 
+static void
+gtv_wire_plugin_vis (gtk_video_private_t *priv, xine_audio_port_t *audio_port)
+{
+  if (!priv->vis_plugin)
+    return;
+
+  if (priv->post_audio)
+    xine_post_wire (gtv_post_output_audio (priv->vis_plugin),
+		    gtv_post_input_audio (priv->post_audio->data));
+  else
+    xine_post_wire_audio_port (gtv_post_output_audio (priv->vis_plugin),
+			       audio_port);
+  xine_post_wire (xine_get_audio_source (priv->stream),
+		  gtv_post_input_audio (priv->vis_plugin));
+}
+
+static void
+gtv_unwire_plugin_vis (gtk_video_private_t *priv, xine_audio_port_t *audio_port)
+{
+  if (!priv->vis_plugin)
+    return;
+
+  if (priv->post_audio)
+    xine_post_wire (xine_get_audio_source (priv->stream),
+		    gtv_post_input_audio (priv->post_audio->data));
+  else
+    xine_post_wire_audio_port (xine_get_audio_source (priv->stream),
+			       audio_port);
+  xine_post_wire (NULL, gtv_post_input_audio (priv->vis_plugin));
+}
+
 static void gtv_recreate_vis (gtk_video_private_t *priv,
 			      xine_audio_port_t **audio_port)
 {
@@ -1577,11 +1608,11 @@ gboolean gtk_video_select_vis (GtkVideo *gtv, const char *id,
   if (priv->vis_plugin_id && id && !strcasecmp (priv->vis_plugin_id, id))
     return priv->vis_plugin != NULL; /* same plugin => nothing to do */
 
-  gtv_unwire_plugins_audio (priv, *audio_port);
+  gtv_unwire_plugin_vis (priv, *audio_port);
   free (priv->vis_plugin_id);
   priv->vis_plugin_id = id ? strdup (id) : NULL;
   gtv_recreate_vis (priv, audio_port);
-  gtv_wire_plugins_audio (priv, *audio_port);
+  gtv_wire_plugin_vis (priv, *audio_port);
 
   return priv->vis_plugin != NULL;
 }
@@ -1600,9 +1631,9 @@ gboolean gtk_video_set_vis (GtkVideo *gtv, xine_audio_port_t **audio_port,
     return priv->vis_plugin != NULL; /* same state => nothing to do */
 
   priv->vis_active = active;
-  gtv_unwire_plugins_audio (priv, *audio_port);
+  gtv_unwire_plugin_vis (priv, *audio_port);
   gtv_recreate_vis (priv, audio_port);
-  gtv_wire_plugins_audio (priv, *audio_port);
+  gtv_wire_plugin_vis (priv, *audio_port);
 
   return priv->vis_plugin != NULL;
 }
