@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: player.h,v 1.2 2004/11/08 01:48:06 dsalt Exp $
+ * $Id: player.h,v 1.3 2005/01/12 00:26:33 dsalt Exp $
  *
  * xine engine wrapper
  */
@@ -25,9 +25,10 @@
 
 void player_init (void);
 
-char *player_get_cur_mrl (void);
+const char *player_get_cur_title (void);
+const char *player_get_cur_mrl (void);
 
-void player_launch (const char *mrl, int pos, int pos_time); 
+void player_launch (const char *tiele, const char *mrl, int pos, int pos_time);
 
 int player_live_stream (void);
 
