@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: playlist.c,v 1.93 2004/12/17 00:33:00 dsalt Exp $
+ * $Id: playlist.c,v 1.112.2.9 2005/07/14 23:57:45 dsalt Exp $
  *
  * playlist implementation
  *
@@ -40,6 +40,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
+#include "xmlparser.h"
 
 #include <pthread.h>
 
@@ -49,8 +50,7 @@
 #include "play_item.h"
 #include "engine.h"
 #include "player.h"
-
-#include "xmlparser.h"
+#include "drag_drop.h"
 
 /*
 #define LOG
@@ -63,10 +63,12 @@ static GtkWidget      *repeat_button, *random_button;
 static char            logo_mrl[1024];
 static int             logo_mode;
 static int             cur_list_pos;
+static int	       ref_list_pos;
 
 #define unmark_play_item() set_mark_play_item (FALSE)
 #define mark_play_item()   set_mark_play_item (TRUE)
 static void set_mark_play_item (gboolean state);
+static void play_next (void);
 
 static inline void remove_trailing_cr (char *str)
 {
@@ -78,11 +80,14 @@ static inline void remove_trailing_cr (char *str)
 static int item_marked_current (GtkTreeIter *iter)
 {
   const char *p;
-  GValue v = { 0 };
+  GValue v;
+  int ret;
+  memset (&v, 0, sizeof (GValue));
   gtk_tree_model_get_value (GTK_TREE_MODEL (pl_store), iter, 3, &v);
   p = g_value_peek_pointer (&v);
+  ret = p && *p;
   g_value_unset (&v);
-  return p && *p;
+  return ret;
 }
 
 static gboolean close_cb (GtkWidget* widget, gpointer data) {
@@ -92,8 +97,10 @@ static gboolean close_cb (GtkWidget* widget, gpointer data) {
   return TRUE;
 }
 
-void playlist_clear (void) {
-  se_eval (gse, "player.stop()", NULL, NULL, NULL);
+void playlist_clear (void)
+{
+  if (!logo_mode)
+    se_eval (gse, "stop()", NULL, NULL, NULL);
   gtk_list_store_clear (pl_store);
 }
 
@@ -120,6 +127,9 @@ static void del_cb (GtkWidget* widget, gpointer data) {
     indices = gtk_tree_path_get_indices (path);
     pos = indices[0];
 
+    if (pos == cur_list_pos && !logo_mode)
+      play_next ();
+
     if (pos <= cur_list_pos)
       --cur_list_pos;
     gtk_list_store_remove (pl_store, &iter);
@@ -153,98 +163,68 @@ static void add_cb (GtkWidget* widget, gpointer data) {
   }
 }
 
-static void up_cb (GtkWidget* widget, gpointer data) {
-
-  GtkTreeIter        iter;  
+static void up_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter        iter;
   GtkTreeSelection  *sel;
 
   sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
 
-  if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    gint        *indices;
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
     GtkTreePath *path;
+    GtkTreeIter  prev;
     int          pos;
-    play_item_t *item;
-    char         str[20];
-    int          marked;
 
     path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
-    indices = gtk_tree_path_get_indices (path);
-    pos = indices[0];
+    pos = gtk_tree_path_get_indices (path) [0];
     gtk_tree_path_free (path);
 
     logprintf ("playlist: got selection, pos=%d\n", pos);
 
-    if (pos<1)
+    if (pos < 1)
       return;
 
-    item = playlist_get_item (pos);
-
     if (pos == cur_list_pos)
       --cur_list_pos; /* we're moving the item being played */
     else if (pos == cur_list_pos + 1)
       ++cur_list_pos; /* we're swapping the item with the one being played */
 
-    /* FIXME: use gtk_list_store_swap () */
-    marked = item_marked_current (&iter);
-    gtk_list_store_remove (GTK_LIST_STORE (pl_store), &iter);
-    playlist_add (item, --pos);
-    if (marked)
-      mark_play_item ();
-
-    snprintf (str, 20, "%d", pos);
-    path = gtk_tree_path_new_from_string  (str);
-    gtk_tree_selection_select_path (sel, path);
-    gtk_tree_path_free (path);
-
+    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &prev, NULL,
+				   pos - 1);
+    gtk_list_store_swap (GTK_LIST_STORE(pl_store), &iter, &prev);
   } else
     logprintf ("playlist: got no selection\n");
 }
 
-static void down_cb (GtkWidget* widget, gpointer data) {
-
-  GtkTreeIter        iter;  
+static void down_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter        iter;
   GtkTreeSelection  *sel;
 
   sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
 
-  if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    gint        *indices;
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
     GtkTreePath *path;
+    GtkTreeIter  next = iter;
     int          pos;
-    play_item_t *item;
-    char         str[20];
-    int          marked;
 
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
-    indices = gtk_tree_path_get_indices (path);
-    pos = indices[0];
+    path = gtk_tree_model_get_path (GTK_TREE_MODEL(pl_store), &iter);
+    pos = gtk_tree_path_get_indices (path) [0];
     gtk_tree_path_free (path);
 
     logprintf ("playlist: got selection, pos=%d\n", pos);
 
-    if (pos>=(playlist_size()-1))
+    if (!gtk_tree_model_iter_next (GTK_TREE_MODEL(pl_store), &next))
       return;
 
-    item = playlist_get_item (pos);
-
     if (pos == cur_list_pos)
       ++cur_list_pos; /* we're moving the item being played */
     else if (pos == cur_list_pos - 1)
       --cur_list_pos; /* we're swapping the item with the one being played */
 
-    /* FIXME: use gtk_list_store_swap () */
-    marked = item_marked_current (&iter);
-    gtk_list_store_remove (GTK_LIST_STORE (pl_store), &iter);
-    playlist_add (item, ++pos);
-    if (marked)
-      mark_play_item ();
-
-    snprintf (str, 20, "%d", pos);
-    path = gtk_tree_path_new_from_string  (str);
-    gtk_tree_selection_select_path (sel, path);
-    gtk_tree_path_free (path);
-
+    gtk_list_store_swap (GTK_LIST_STORE(pl_store), &iter, &next);
   } else
     logprintf ("playlist: got no selection\n");
 }
@@ -252,10 +232,8 @@ static void down_cb (GtkWidget* widget, gpointer data) {
 static void edit_cb (GtkWidget* widget, gpointer data) {
 
   GtkTreeView       *tree = (GtkTreeView *) data;
-  GtkTreeSelection  *sel;
-  GtkTreeIter        iter;  
-
-  sel = gtk_tree_view_get_selection (tree);
+  GtkTreeSelection  *sel = gtk_tree_view_get_selection (tree);
+  GtkTreeIter        iter;
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
     GValue       v;
@@ -270,6 +248,7 @@ static void edit_cb (GtkWidget* widget, gpointer data) {
 	    ? xine_get_meta_info (stream, XINE_META_INFO_TITLE) : NULL;
 
     if (play_item_edit (play_item, PLAY_ITEM_LIST, title))
+      /* FIXME: what if the MRL is changed? */
       gtk_list_store_set (pl_store, &iter,
 			  0, play_item->title,
 			  1, play_item->mrl, 2, play_item, -1);
@@ -303,8 +282,8 @@ static int playlist_load (char *fname) {
 
 	while (node) {
 
-	  if (!strcasecmp (node->name, "entry")) {
-
+	  if (!strcasecmp (node->name, "entry"))
+	  {
 	    play_item_t *play_item = play_item_load (node->child);
 	    GtkTreeIter  iter;
 
@@ -313,6 +292,15 @@ static int playlist_load (char *fname) {
 	    gtk_list_store_set (pl_store, &iter, 0, play_item->title, 1, 
 				play_item->mrl, 2, play_item, 3, "", -1);
 	  }
+	  else if (!strcasecmp (node->name, "settings"))
+	  {
+	    gtk_toggle_button_set_active
+	      (GTK_TOGGLE_BUTTON (random_button),
+	       xml_parser_get_property_bool (node, "RANDOM", 0));
+	    gtk_toggle_button_set_active
+	      (GTK_TOGGLE_BUTTON (repeat_button),
+	       xml_parser_get_property_bool (node, "REPEAT", 0));
+	  }
 
 	  node = node->next;
 	}
@@ -347,25 +335,26 @@ void playlist_save (char *fname) {
     fname = g_strconcat(g_get_home_dir(), "/.gxine/playlist", NULL);
 
   f = fopen (fname, "w");
-  if (f) {
+  if (f)
+  {
+    static const char tf[][8] = { "false", "true" };
 
     GtkTreeIter iter;
 
-    fprintf (f, "<ASX VERSION=\"3.0\">\n");
+    fprintf
+      (f, "<ASX VERSION=\"3.0\">\n"
+          "  <SETTINGS REPEAT=\"%s\" RANDOM=\"%s\"/>\n",
+       tf[!!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(repeat_button))],
+       tf[!!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(random_button))]);
 
     if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (pl_store), &iter)) {
 
       do {
-
 	GValue vi;
-	play_item_t  *play_item;
-
 	memset (&vi, 0, sizeof (GValue));
 	gtk_tree_model_get_value (GTK_TREE_MODEL (pl_store),
 				  &iter, 2, &vi);
-	play_item = g_value_peek_pointer (&vi);
-	
-	play_item_save (play_item, f, 2);
+	play_item_save (g_value_peek_pointer (&vi), f, 1);
 
 	g_value_unset (&vi);
       } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(pl_store), &iter));
@@ -627,8 +616,6 @@ int playlist_add (play_item_t *play_item, gint ins_pos) {
 static gint playlist_add_int (const char *mrl, gint ins_pos)
 {
   play_item_t *play_item;
-  const char *title;
-  char *retitle = NULL;
   int i;
 
   /* don't add duplicate items */
@@ -637,55 +624,7 @@ static gint playlist_add_int (const char *mrl, gint ins_pos)
       if (!strcmp (playlist_get_item (i)->mrl, mrl))
         return i;
 
-  /* find the leafname of the MRL */
-  title = strrchr (mrl, '/');
-  if (title && !title[1]) /* ends in '/' - look for preceding '/' */
-  {
-    while (*title == '/')
-      if (--title == mrl)
-        break;
-    while (*title != '/')
-      if (--title < mrl)
-        break;
-  }
-
-  /* at this point, title >= mrl - 1, but mrl[-1] is invalid - this is OK
-   * in the case where title == mrl - 1, we use the full MRL as the title
-   */
-
-  if (title >= mrl && title[1])
-  {
-    int j;
-    char *dot;
-
-    /* URL-decode the leafname */
-    title = retitle = strdup (title + 1);
-    j = i = -1;
-    while (title[++i])
-    {
-      if (title[i] == '%')
-      {
-	char c[3] = { title[i+1], title[i+2], 0 };
-	i += 2;
-	retitle[++j] = (char) strtol (c, NULL, 16);
-      }
-      else
-        retitle[++j] = title[i];
-    }
-    /* NUL-terminate, strip trailing / */
-    retitle[j + (retitle[j] != '/')] = 0;
-
-    /* Strip the extension (if present) */
-    dot = strrchr (retitle, '.');
-    if (dot)
-      *dot = 0;
-  }
-  else
-    title = mrl;
-
-  play_item = play_item_new (title, mrl, 0);
-  free (retitle);
-
+  play_item = play_item_new (NULL, mrl, 0);
   return playlist_add (play_item, ins_pos);
 }
 
@@ -927,12 +866,8 @@ static int playlist_load_asx (const char *mrl, gint ins_pos) {
 		logprintf("playlist: HREF property detected\n"
 			  "playlist: found an mrl: %s\n", property->value);
 
-		if (title)
-		  item = play_item_new (title, property->value, 0);
-		else
-		  item = play_item_new (property->value, property->value, 0);
-		
-		
+		item = play_item_new (title, property->value, 0);
+
 		if (ret >= 0)
 		  playlist_add (item, ins_pos);
 		else
@@ -1179,7 +1114,7 @@ void playlist_logo (void) {
   unmark_play_item ();
   logo_mode = 1;
 
-  player_launch (logo_mrl, 0, 0);
+  player_launch (NULL, logo_mrl, 0, 0);
 }
 
 void playlist_show (void) {
@@ -1199,37 +1134,21 @@ int playlist_showing_logo (void)
   return logo_mode;
 }
 
-static gboolean button_press_lcb (GtkWidget *widget, GdkEventButton *event, 
-				  gpointer data) {
-  if (event && (event->type==GDK_2BUTTON_PRESS)) {
-    GtkTreeSelection  *sel;
-    GtkTreeIter        iter;  
+static gboolean row_activated_lcb (GtkWidget *widget, GtkTreePath *path,
+				   GtkTreeViewColumn *col, gpointer data)
+{
+  GtkTreeSelection *sel;
+  GtkTreeIter iter;
 
-    sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
+  sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+    playlist_play (gtk_tree_path_get_indices (path)[0]);
 
-    if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-
-      GtkTreePath *path;
-      gint        *indices, item;
-
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
-      indices = gtk_tree_path_get_indices (path);
-      item = indices[0];
-      gtk_tree_path_free (path);
-
-      playlist_play (item); 
-    }
-  }
   return FALSE;
 }
 
 static int has_been_played (int pos) {
-
-  play_item_t *item;
-
-  item = playlist_get_item (pos);
-
-  return item->played;
+  return playlist_get_item (pos)->played;
 }
 
 static void play_next (void) {
@@ -1240,6 +1159,7 @@ static void play_next (void) {
   is_random = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (random_button));
 
   unmark_play_item ();
+  ref_list_pos = -1;
 
   if (is_random) {
 
@@ -1300,15 +1220,12 @@ static void xine_event_cb (void *user_data, const xine_event_t *event) {
 
   switch (event->type) {
   case XINE_EVENT_UI_PLAYBACK_FINISHED:
-
     gdk_threads_enter();
-    if (!logo_mode) {
-
+    if (!logo_mode)
       play_next ();
-
-    }
     gdk_threads_leave();
     break;
+
   case XINE_EVENT_PROGRESS:
     {
       xine_progress_data_t *prg = event->data;
@@ -1334,32 +1251,48 @@ static void xine_event_cb (void *user_data, const xine_event_t *event) {
       gdk_threads_leave();
     }
     break;
+
   case XINE_EVENT_UI_SET_TITLE:
     {
       gdk_threads_enter();
       infobar_show_metadata ();
+      playlist_check_set_title ();
       gdk_flush();
       gdk_threads_leave();
     }
     break;
+
   case XINE_EVENT_MRL_REFERENCE:
     {
       xine_mrl_reference_data_t *ref = (xine_mrl_reference_data_t *) event->data;
 
       logprintf ("playlist: got a reference event (ref MRL: %s)\n",
 	      ref->mrl);
-      
-      if (ref->alternative == 0) {
 
-	play_item_t *item;
+      if (ref->alternative == 0)
+      {
+	play_item_t *item = play_item_new (NULL, ref->mrl, 0);
+	gdk_threads_enter ();
 
-	item = play_item_new (ref->mrl, ref->mrl, 0);
+	if (ref_list_pos == -1)
+	{
+	  GtkTreeIter iter;
+	  ref_list_pos = cur_list_pos + 1;
+	  playlist_add (item, ref_list_pos);
+	  /* remove the item which triggered this reference event */
+	  gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter,
+					 NULL, cur_list_pos--);
+	  gtk_list_store_remove (pl_store, &iter);
+	}
+	else
+	  playlist_add (item, ++ref_list_pos);
 
-	playlist_add (item, playlist_get_list_pos()+1);
+	gdk_threads_leave ();
       }
     }
     break;
-  case XINE_EVENT_UI_MESSAGE: 
+
+  case XINE_EVENT_UI_MESSAGE:
     {
       xine_ui_message_data_t *data = (xine_ui_message_data_t *) event->data;
       void (*msg)(const gchar *, const gchar *, ...);
@@ -1411,85 +1344,122 @@ static void xine_event_cb (void *user_data, const xine_event_t *event) {
       gdk_threads_leave();
     }
     break;
-    
-  }
 
+  case XINE_EVENT_AUDIO_LEVEL:
+    {
+      xine_audio_level_data_t *data = event->data;
+      gdk_threads_enter ();
+      ui_set_control_adjustment (Control_VOLUME,
+				 (data->left + data->right) / 2);
+      ui_set_control_button (Control_MUTE, !data->mute);
+      gdk_threads_leave ();
+    }
+  }
+}
+
+void playlist_check_set_title (void)
+{
+  const char *title;
+  if ((title = xine_get_meta_info (stream, XINE_META_INFO_TITLE)))
+  {
+    GtkTreeIter iter;
+    play_item_t *play_item;
+    GValue v;
+
+    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter,
+				   NULL, cur_list_pos);
+
+    memset (&v, 0, sizeof (GValue));
+    gtk_tree_model_get_value (GTK_TREE_MODEL (pl_store), &iter, 2, &v);
+    play_item = g_value_peek_pointer (&v);
+
+    if (play_item->untitled)
+    {
+      free (play_item->title);
+      play_item->title = strdup (title);
+      play_item->untitled = FALSE;
+      gtk_list_store_set (pl_store, &iter, 0, play_item->title, -1);
+    }
+
+    g_value_unset (&v);
+  }
 }
 
 /*
  * drag and drop
+ * FIXME: should subclass GtkListStore
  */
 
-static const GtkTargetEntry target_table[] = {
-	{ "text/uri-list", 0, 0 },
-};
-
-static void drop_cb (GtkWidget          *widget,
+static void drop_cb (GtkTreeView	*widget,
 		     GdkDragContext     *context,
 		     gint                x,
 		     gint                y,
 		     GtkSelectionData   *data,
 		     guint               info,
-		     guint               time) {
+		     guint		 time)
+{
+  gint ins_pos = treeview_get_drop_index (widget, context, x, y);
 
-  GtkTreePath *path;
-  gint         cx, cy;
-  gint         ins_pos;
+  logprintf ("drag_drop: drop callback, length=%d, format=%d, pos=%d,%d, insert=%d\n",
+	     data->length, data->format, x, y, ins_pos);
 
-  logprintf ("drag_drop: drop callback, length=%d, format=%d\n, x=%d, y=%d\n",
-	     data->length, data->format, x, y);
-
-  if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (tree_view), x, y, &path, 
-				     NULL, &cx, &cy)) {
-    
-    gint        *indices;
-    
-    indices = gtk_tree_path_get_indices (path);
-    ins_pos = indices[0];
-
-    logprintf ("playlist: mapping to tree view worked: cx=%d, cy=%d, path=%s, row=%d\n",
-	       cx, cy, gtk_tree_path_to_string (path), ins_pos);
-
-    gtk_tree_path_free (path);
-  } else
-    ins_pos = -1;
-    
-
-  if (data->format == 8) {
-    
-    gint  i, len;
-    gchar str[MAX_MRL_LEN];
-    
-    len = 0; i = 0;
-
-    while (i<data->length) {
-
-      char c;
-
-      c = data->data[i];
-
-      if ( (len<(MAX_MRL_LEN-1)) 
-	   && (c != '\r') 
-	   && (c != '\n') ) {
-	str[len] = c;
-	len++;
-      } else if (c=='\r') {
-
-	str[len] = 0;
-
-	playlist_add_mrl (str, ins_pos);
-
-	if (ins_pos >= 0 && ins_pos++ <= cur_list_pos)
-	  ++cur_list_pos;
-
-	len = 0;
-
-      } 
-      i++;
-    }
+  if (data->format == 8)
+  {
+    /* Drag is from an external source */
+    dnd_add_mrls (data, &cur_list_pos, ins_pos);
+    gtk_drag_finish (context, TRUE, FALSE, time);
+    return;
   }
 
-  gtk_drag_finish (context, TRUE, FALSE, time);
+  if (drag_mrl.index == -1)
+  {
+    /* below here, we're only interested in intra-app drags */
+    gtk_drag_finish (context, FALSE, FALSE, time);
+    return;
+  }
+
+  if (drag_mrl.model == (GtkTreeModel *)pl_store)
+  {
+    /* Moving a playlist item */
+    GtkTreeIter from, to;
+
+    if (drag_mrl.index == ins_pos)
+    {
+      /* no need to move the item */
+      gtk_drag_finish (context, TRUE, FALSE, time);
+      return;
+    }
+
+    gtk_tree_model_iter_nth_child (drag_mrl.model, &from, NULL, drag_mrl.index);
+    if (ins_pos == -1)
+    {
+      gtk_list_store_move_before (pl_store, &from, NULL);
+      ins_pos = gtk_tree_model_iter_n_children (drag_mrl.model, NULL);
+    }
+    else
+    {
+      gtk_tree_model_iter_nth_child (drag_mrl.model, &to, NULL, ins_pos);
+      gtk_list_store_move_before (pl_store, &from, &to);
+    }
+
+    /* adjust index of currently-being-played item if necessary */
+    if (cur_list_pos != -1)
+    {
+      if (cur_list_pos == drag_mrl.index)
+	cur_list_pos = ins_pos;
+      if (drag_mrl.index < ins_pos && drag_mrl.index <= cur_list_pos
+	  && cur_list_pos <= ins_pos)
+	--cur_list_pos;
+      if (drag_mrl.index > ins_pos && drag_mrl.index >= cur_list_pos
+	  && cur_list_pos >= ins_pos)
+	++cur_list_pos;
+    }
+
+    gtk_drag_finish (context, TRUE, FALSE, time);
+    return;
+  }
+
+  gtk_drag_finish (context, FALSE, FALSE, time);
 }
 
 /*
@@ -1602,7 +1572,7 @@ void playlist_play_from (int list_pos, int pos, int pos_time) {
     return;
   }
 
-  play_item_play (play_item);
+  play_item_play (play_item, pos, pos_time);
   
   snprintf (indices, 10, "%d", list_pos);
   path = gtk_tree_path_new_from_string (indices);
@@ -1610,10 +1580,10 @@ void playlist_play_from (int list_pos, int pos, int pos_time) {
 			    path, NULL, FALSE);
   gtk_tree_path_free (path);
 
-  cur_list_pos  = list_pos;
+  cur_list_pos = list_pos;
+  ref_list_pos = -1;
+
   mark_play_item ();
-
-
 }
 
 void playlist_init (void) {
@@ -1634,7 +1604,7 @@ void playlist_init (void) {
   dlg = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dlg), _("Edit playlist..."));
   gtk_window_set_default_size (GTK_WINDOW (dlg), 500, 400);
-  g_signal_connect (GTK_OBJECT (dlg), "delete_event",
+  g_signal_connect (GTK_OBJECT (dlg), "delete-event",
 		    G_CALLBACK (close_cb), NULL );
 
   /*
@@ -1675,8 +1645,8 @@ void playlist_init (void) {
 
   tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pl_store));  
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
-  g_signal_connect (G_OBJECT(tree_view), "button_press_event",
-		    G_CALLBACK(button_press_lcb), NULL);
+  g_signal_connect (G_OBJECT(tree_view), "row-activated",
+		    G_CALLBACK(row_activated_lcb), NULL);
 
   cell = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (" ", cell,
@@ -1684,25 +1654,32 @@ void playlist_init (void) {
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
-  cell = gtk_cell_renderer_text_new ();
+  if (GTK_TEST_COMPAT_2_6)
+  {
+    cell = gtk_cell_renderer_text_new ();
+    g_object_set (G_OBJECT(cell),
+		  "ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
+  }
   column = gtk_tree_view_column_new_with_attributes (_("Title"), cell,
 						     "text", 0, NULL);
+  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
-  column = gtk_tree_view_column_new_with_attributes ("MRL", cell,
+  if (GTK_TEST_COMPAT_2_6)
+  {
+    cell = gtk_cell_renderer_text_new ();
+    g_object_set (G_OBJECT(cell),
+		  "ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
+  }
+  column = gtk_tree_view_column_new_with_attributes (_("MRL"), cell,
 						     "text", 1, NULL);
+  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
+				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
-  /* FIXME: interacts badly with own drag&drop stuff 
-     gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree_view), TRUE);
-  */
-
-  g_signal_connect (G_OBJECT (tree_view), "drag_data_received",
-		    G_CALLBACK (drop_cb), NULL);
-  gtk_drag_dest_set (tree_view, GTK_DEST_DEFAULT_ALL,
-		     target_table, 1, GDK_ACTION_COPY);
+  treeview_drag_drop_setup (GTK_TREE_VIEW(tree_view), drop_cb);
 
   gtk_widget_show (tree_view);
 
@@ -1712,7 +1689,6 @@ void playlist_init (void) {
 				  GTK_POLICY_AUTOMATIC);
   gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
 
-  
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG (dlg)->vbox), scrolled_window,
 		      TRUE, TRUE, 2);
 
@@ -1722,15 +1698,14 @@ void playlist_init (void) {
 
   hbox = gtk_hbox_new (0,2);
   
-  repeat_button = gtk_check_button_new_with_label (_("Repeat"));
+  repeat_button = gtk_check_button_new_with_mnemonic (_("_Repeat"));
   gtk_box_pack_start (GTK_BOX (hbox), repeat_button, FALSE, FALSE, 2);
 
-  random_button = gtk_check_button_new_with_label (_("Random"));
+  random_button = gtk_check_button_new_with_mnemonic (_("_Random"));
   gtk_box_pack_start (GTK_BOX (hbox), random_button, FALSE, FALSE, 2);
 
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hbox,
 		      FALSE, FALSE, 2);
-  
 
   /*
    * button bar
@@ -1743,7 +1718,8 @@ void playlist_init (void) {
 		    G_CALLBACK(add_cb), 
 		    tree_view);
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 2);
-  button = gtk_button_new_with_label (_("Edit"));
+  button = GTK_COMPAT_2_6 (gtk_button_new_from_stock ("gtk-edit"),
+			   gtk_button_new_with_mnemonic (_("_Edit")));
   g_signal_connect (GTK_OBJECT(button), "clicked", 
 		    G_CALLBACK(edit_cb), 
 		    tree_view);
@@ -1772,12 +1748,11 @@ void playlist_init (void) {
 		    G_CALLBACK(close_cb), 
 		    NULL);
 
-  playlist_load (NULL);
-
   sprintf (logo_mrl, "%s/logo.mpv", logodir);
-
   logo_mode = 1;
 
+  playlist_load (NULL);
+
   /*
    * event handling
    */
