@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: post.c,v 1.20.2.2 2005/05/19 00:25:43 dsalt Exp $
+ * $Id: post.c,v 1.20.2.3 2005/08/01 22:47:17 dsalt Exp $
  *
  * post-plugin handling
  */
@@ -432,8 +432,8 @@ char_entry_cb (GtkEditable *widget, GdkEventFocus *event,
 {
   const gxine_post_t *info = g_object_get_data (G_OBJECT(widget), "post");
   memset (info->params + param->offset, 0, param->size);
-  snprintf (info->params + param->offset, param->size,
-	    gtk_editable_get_chars (widget, 0, -1));
+  strncpy (info->params + param->offset, gtk_editable_get_chars (widget, 0, -1),
+	   param->size - 1);
   return FALSE;
 }
 
