@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004 the xine-project
+ * Copyright (C) 2004-2005 the xine-project
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: time_widget.h,v 1.1 2004/04/09 21:41:01 guenter Exp $
+ * $Id: time_widget.h,v 1.2 2005/01/12 18:45:04 dsalt Exp $
  *
  * a nice black time display area
  */
@@ -29,7 +29,6 @@ typedef struct time_widget_s time_widget_t;
 
 time_widget_t *create_time_widget (GtkWidget **w);
 
-void postinit_time_widget (GtkWidget *window, time_widget_t *tw) ;
+void postinit_time_widget (time_widget_t *tw);
 
 #endif
-
