@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  *
- * $Id: utils.c,v 1.50.2.3 2005/07/06 19:13:45 dsalt Exp $
+ * $Id: utils.c,v 1.50.2.7 2005/09/12 00:45:46 dsalt Exp $
  *
  * needful things, and various UI bits:
  * - button registry and update (state display etc.)
@@ -112,6 +112,9 @@ char *read_entire_file (const char *mrl, int *file_size) {
   {
     /* read error */
     free (buf);
+    size = errno;
+    close (fd);
+    errno = size;
     return NULL;
   }
 
@@ -554,16 +557,16 @@ typedef struct {
 } ui_adjustable_t;
 
 static const ui_adjustable_t ranges[] = {
-  [Control_SEEKER]		= {     0,       0,  65535,    1,    10, 1, NULL },
-  [Control_AUDIO_CHANNEL]	= {    -1,      -1,     32,    1,     1, 1, NULL,		 XINE_PARAM_AUDIO_CHANNEL_LOGICAL },
-  [Control_VOLUME]		= {    50,       0,    100,    1,    10, 1, "audio.volume.mixer_volume", XINE_PARAM_AUDIO_VOLUME },
-  [Control_COMPRESSOR]		= {   100,     100,   1000,   10,   100, 1, "gui.ao_compressor", XINE_PARAM_AUDIO_COMPR_LEVEL },
-  [Control_AMPLIFIER]		= {   100,       0,    200,    1,    10, 1, "gui.ao_amplifier",  XINE_PARAM_AUDIO_AMP_LEVEL },
-  [Control_AV_SYNC]		= {     0, -180000, 180000, 1000, 10000, 1, "gui.av_sync",	 XINE_PARAM_AV_OFFSET },
-  [Control_HUE]			= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_hue",	 XINE_PARAM_VO_HUE },
-  [Control_SATURATION]		= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_saturation", XINE_PARAM_VO_SATURATION },
-  [Control_CONTRAST]		= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_contrast",   XINE_PARAM_VO_CONTRAST },
-  [Control_BRIGHTNESS]		= { 32768,       0,  65535,  100,  1000, 1, "gui.vo_brightness", XINE_PARAM_VO_BRIGHTNESS },
+  [Control_SEEKER]		= {     0,       0,  65535,    1,    10, 0, NULL },
+  [Control_AUDIO_CHANNEL]	= {    -1,      -1,     32,    1,     1, 0, NULL,		 XINE_PARAM_AUDIO_CHANNEL_LOGICAL },
+  [Control_VOLUME]		= {    50,       0,    100,    1,    10, 0, "audio.volume.mixer_volume", XINE_PARAM_AUDIO_VOLUME },
+  [Control_COMPRESSOR]		= {   100,     100,   1000,   10,   100, 0, "gui.ao_compressor", XINE_PARAM_AUDIO_COMPR_LEVEL },
+  [Control_AMPLIFIER]		= {   100,       0,    200,    1,    10, 0, "gui.ao_amplifier",  XINE_PARAM_AUDIO_AMP_LEVEL },
+  [Control_AV_SYNC]		= {     0, -180000, 180000, 1000, 10000, 0, "gui.av_sync",	 XINE_PARAM_AV_OFFSET },
+  [Control_HUE]			= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_hue",	 XINE_PARAM_VO_HUE },
+  [Control_SATURATION]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_saturation", XINE_PARAM_VO_SATURATION },
+  [Control_CONTRAST]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_contrast",   XINE_PARAM_VO_CONTRAST },
+  [Control_BRIGHTNESS]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_brightness", XINE_PARAM_VO_BRIGHTNESS },
 };
 static gdouble starts[sizeof (ranges) / sizeof (ranges[0])];
 static gdouble inits[sizeof (ranges) / sizeof (ranges[0])];
@@ -613,14 +616,10 @@ GtkObject *ui_register_control_adjustment (control_adjustment_t item)
     starts[item] = start;
 
     if (ranges[item].param)
-    {
       inits[item] = start = xine_get_param (stream, ranges[item].param);
-      if (item != Control_VOLUME)
-        xine_set_param (stream, ranges[item].param, start);
-    }
 
     c_adjustments[item] =
-      gtk_adjustment_new (start, ranges[item].min, ranges[item].max + 1,
+      gtk_adjustment_new (start, ranges[item].min, ranges[item].max,
 			  ranges[item].step, ranges[item].page,
 			  ranges[item].pagesize);
     if (ranges[item].param || ranges[item].setting)
@@ -861,3 +860,14 @@ void ui_init (void)
     toolbar_visible = (entry.num_value >> 1) & 1;
   }
 }
+
+gboolean ui_post_init (void)
+{
+  int i;
+  gdk_threads_enter ();
+  for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
+    if (ranges[i].param && i != Control_VOLUME)
+      xine_set_param (stream, ranges[i].param, starts[i]);
+  gdk_threads_leave ();
+  return FALSE;
+}
