@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: vis.h,v 1.1 2004/11/17 02:12:07 dsalt Exp $
+ * $Id: vis.h,v 1.3.2.1 2005/04/29 20:01:45 dsalt Exp $
  *
  * Visualisation wrapper code
  */
@@ -23,9 +23,9 @@
 #ifndef HAVE_VIS_H
 #define HAVE_VIS_H
 
-void vis_hide (GtkVideo *, xine_audio_port_t *);
+#define vis_hide(V,A) gtk_video_set_vis ((V), (A), FALSE)
 gboolean vis_show (GtkVideo *, xine_audio_port_t **);
-void vis_set (GtkVideo *, const char *id, xine_audio_port_t **);
+void vis_set (GtkVideo *, const char *, xine_audio_port_t **);
 
 void vis_init (void);
 
