@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 the xine project
+ *  Copyright (C) 2002-2003 the xine project
  *
  *  This file is part of xine, a free video player.
  *
@@ -17,26 +17,27 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- *  $Id: xmllexer.c,v 1.4 2004/11/05 21:43:39 dsalt Exp $
+ *  $Id: xmllexer.c,v 1.6.2.1 2005/03/08 23:34:44 dsalt Exp $
  *
  */
 
-#include "config.h"
-#include "defs.h"
+#define LOG_MODULE "xmllexer"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
 
+#include "globals.h"
 #include "xmllexer.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <stdlib.h>
 
 /* private constants*/
 #define NORMAL       0  /* normal lex mode */
 #define DATA         1  /* data lex mode */
 
-/*
-#define LOG
-*/
-
 /* private global variables */
 static char * lexbuf;
 static int lexbuf_size = 0;
@@ -50,7 +51,8 @@ void lexer_init(char * buf, int size) {
   lexbuf_pos  = 0;
   lex_mode    = NORMAL;
   in_comment  = 0;
-  logprintf("xmllexer: buffer length %d\n", size);
+
+  logprintf("buffer length %d\n", size);
 }
 
 int lexer_get_token(char * tok, int tok_size) {
@@ -61,7 +63,8 @@ int lexer_get_token(char * tok, int tok_size) {
   if (tok) {
     while ((tok_pos < tok_size) && (lexbuf_pos < lexbuf_size)) {
       c = lexbuf[lexbuf_pos];
-      logprintf("xmllexer: c=%c, state=%d, in_comment=%d\n", c, state, in_comment);
+      logprintf("c=%c, state=%d, in_comment=%d\n", c, state, in_comment);
+
       if (lex_mode == NORMAL) {
 				/* normal mode */
 	switch (state) {
@@ -107,7 +110,7 @@ int lexer_get_token(char * tok, int tok_size) {
 	    tok_pos++;
 	    break;
 
-	  case '\"':
+	  case '\"': /* " */
 	    state = 7;
 	    break;
 
@@ -219,7 +222,7 @@ int lexer_get_token(char * tok, int tok_size) {
 	case 7:
 	  tok[tok_pos] = c;
 	  lexbuf_pos++;
-	  if (c == '\"') {
+	  if (c == '\"') { /* " */
 	    tok[tok_pos] = '\0'; /* FIXME */
 	    return T_STRING;
 	  }
@@ -319,10 +322,11 @@ int lexer_get_token(char * tok, int tok_size) {
 	  case '<':
 	  case '>':
 	  case '\\':
-	  case '\"':
+	  case '\"': /* " */
 	  case ' ':
 	  case '\t':
 	  case '=':
+	  case '/':
 	    tok[tok_pos] = '\0';
 	    return T_IDENT;
 	    break;
@@ -345,27 +349,30 @@ int lexer_get_token(char * tok, int tok_size) {
 	  }
 	  break;
 	default:
-	  printf("xmllexer: expected char \'%c\'\n", tok[tok_pos - 1]); /* FIX ME */
+	  logprintf("expected char \'%c\'\n", tok[tok_pos - 1]); /* FIX ME */
 	  return T_ERROR;
 	}
       } else {
 				/* data mode, stop if char equal '<' */
-	if (c == '<') {
+        switch (c)
+        {
+        case '<':
 	  tok[tok_pos] = '\0';
 	  lex_mode = NORMAL;
 	  return T_DATA;
-	} else {
+	default:
 	  tok[tok_pos] = c;
 	  tok_pos++;
 	  lexbuf_pos++;
 	}
       }
     }
-    logprintf ("xmllexer: loop done tok_pos = %d, tok_size=%d, lexbuf_pos=%d, lexbuf_size=%d\n",
-	       tok_pos, tok_size, lexbuf_pos, lexbuf_size);
+    logprintf ("loop done tok_pos = %d, tok_size=%d, lexbuf_pos=%d, lexbuf_size=%d\n", 
+	     tok_pos, tok_size, lexbuf_pos, lexbuf_size);
+
     /* pb */
     if (tok_pos >= tok_size) {
-      printf("xmllexer: token buffer is too little\n");
+      logprintf("token buffer is too little\n");
     } else {
       if (lexbuf_pos >= lexbuf_size) {
 				/* Terminate the current token */
@@ -395,15 +402,85 @@ int lexer_get_token(char * tok, int tok_size) {
 	  return T_DATA;
 	  break;
 	default:
-	  printf("xmllexer: unknown state, state=%d\n", state);
+	  logprintf("unknown state, state=%d\n", state);
 	}
       } else {
-	printf("xmllexer: abnormal end of buffer, state=%d\n", state);
+	logprintf("abnormal end of buffer, state=%d\n", state);
       }
     }
     return T_ERROR;
   }
   /* tok == null */
-  printf("xmllexer: token buffer is null\n");
+  logprintf("token buffer is null\n");
   return T_ERROR;
 }
+
+static struct {
+  char code;
+  unsigned char namelen;
+  char name[6];
+} lexer_entities[] = {
+  { '"',  4, "quot" },
+  { '&',  3, "amp" },
+  { '\'', 4, "apos" },
+  { '<',  2, "lt" },
+  { '>',  2, "gt" },
+  { 0 }
+};
+
+char *lexer_decode_entities (char *tok)
+{
+  char *buf = malloc (strlen (tok) + 1);
+  char *bp = buf;
+  char c;
+
+  while ((c = *tok++))
+  {
+    if (c != '&')
+      *bp++ = c;
+    else
+    {
+      /* parse the character entity (on failure, treat it as literal text) */
+      char *tp = tok;
+      long i;
+
+      for (i = 0; lexer_entities[i].code; ++i)
+	if (!strncmp (lexer_entities[i].name, tok, lexer_entities[i].namelen)
+	    && tok[lexer_entities[i].namelen] == ';')
+	  break;
+      if (lexer_entities[i].code)
+      {
+        tok += lexer_entities[i].namelen + 1;
+	*bp++ = lexer_entities[i].code;
+	continue;
+      }
+
+      if (*tp++ != '#')
+      {
+        /* not a recognised name and not numeric */
+	*bp++ = '&';
+	continue;
+      }
+
+      /* entity is a number
+       * (note: strtol() allows "0x" prefix for hexadecimal, but we don't)
+       */
+      if (*tp == 'x' && tp[1] && tp[2] != 'x')
+	i = strtol (tp + 1, &tp, 16);
+      else
+	i = strtol (tp, &tp, 10);
+
+      if (i < 1 || i > 255 || *tp != ';')
+      {
+        /* out of range, or format error */
+	*bp++ = '&';
+	continue;
+      }
+
+      tok = tp + 1;
+      *bp++ = i;
+    }
+  }
+  *bp = 0;
+  return buf;
+}
