@@ -18,18 +18,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- *  $Id: xmlparser.c,v 1.5 2004/11/05 21:43:39 dsalt Exp $
+ *  $Id: xmlparser.c,v 1.7.2.1 2005/03/08 23:34:44 dsalt Exp $
  *
  */
 
-#include "config.h"
-#include "defs.h"
-
+#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <ctype.h>
+
+#define LOG_MODULE "xmlparser"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
+#include "globals.h"
 #include "xmllexer.h"
 #include "xmlparser.h"
 
@@ -38,10 +44,6 @@
 #define DATA_SIZE   4 * 1024
 #define MAX_RECURSION 10
 
-/*
-#define LOG
-*/
-
 /* private global variables */
 static int xml_parser_mode;
 
@@ -57,7 +59,7 @@ static char * strtoupper(char * str) {
   return str;
 }
 
-static xml_node_t * new_xml_node() {
+static xml_node_t * new_xml_node(void) {
   xml_node_t * new_node;
 
   new_node = (xml_node_t*) malloc(sizeof(xml_node_t));
@@ -70,10 +72,14 @@ static xml_node_t * new_xml_node() {
 }
 
 static void free_xml_node(xml_node_t * node) {
+  if (node->name)
+    free (node->name);
+  if (node->data)
+    free (node->data);
   free(node);
 }
 
-static xml_property_t * new_xml_property() {
+static xml_property_t * new_xml_property(void) {
   xml_property_t * new_property;
 
   new_property = (xml_property_t*) malloc(sizeof(xml_property_t));
@@ -84,6 +90,10 @@ static xml_property_t * new_xml_property() {
 }
 
 static void free_xml_property(xml_property_t * property) {
+  if (property->name)
+    free (property->name);
+  if (property->value)
+    free (property->value);
   free(property);
 }
 
@@ -93,7 +103,7 @@ void xml_parser_init(char * buf, int size, int mode) {
   xml_parser_mode = mode;
 }
 
-void xml_parser_free_props(xml_property_t *current_property) {
+static void xml_parser_free_props(xml_property_t *current_property) {
   if (current_property) {
     if (!current_property->next) {
       free_xml_property(current_property);
@@ -104,31 +114,48 @@ void xml_parser_free_props(xml_property_t *current_property) {
   }
 }
 
-void xml_parser_free_tree(xml_node_t *current_node) {
+static void xml_parser_free_tree_rec(xml_node_t *current_node, int free_next) {
+  logprintf("xml_parser_free_tree_rec: %s\n", current_node->name);
+
   if (current_node) {
-    /* propertys */
+    /* properties */
     if (current_node->props) {
       xml_parser_free_props(current_node->props);
     }
 
     /* child nodes */
     if (current_node->child) {
-      xml_parser_free_tree(current_node->child);
+      logprintf("xml_parser_free_tree_rec: child\n");
+      xml_parser_free_tree_rec(current_node->child, 1);
     }
 
-    if (current_node->next) {
-      xml_parser_free_tree(current_node->next);
+    /* next nodes */
+    if (free_next) {
+      xml_node_t *next_node = current_node->next;
+      xml_node_t *next_next_node;
+
+      while (next_node) {
+        next_next_node = next_node->next;
+        logprintf("xml_parser_free_tree_rec: next\n");
+        xml_parser_free_tree_rec(next_node, 0);
+        next_node = next_next_node;
+      }
     }
 
     free_xml_node(current_node);
   }
 }
 
+void xml_parser_free_tree(xml_node_t *current_node) {
+  logprintf("xml_parser_free_tree\n");
+   xml_parser_free_tree_rec(current_node, 1);
+}
+
 #define STATE_IDLE    0
 #define STATE_NODE    1
 #define STATE_COMMENT 7
 
-int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
+static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
   char tok[TOKEN_SIZE];
   char property_name[TOKEN_SIZE];
   char node_name[TOKEN_SIZE];
@@ -139,13 +166,14 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
   xml_node_t *subtree = NULL;
   xml_node_t *current_subtree = NULL;
   xml_property_t *current_property = NULL;
-  xml_property_t *propertys = NULL;
+  xml_property_t *properties = NULL;
 
   if (rec < MAX_RECURSION) {
 
     while ((bypass_get_token) || (res = lexer_get_token(tok, TOKEN_SIZE)) != T_ERROR) {
       bypass_get_token = 0;
-      logprintf("xmlparser: info: %d - %d : '%s'\n", state, res, tok);
+      logprintf("info: %d - %d : '%s'\n", state, res, tok);
+
       switch (state) {
       case STATE_IDLE:
 	switch (res) {
@@ -173,11 +201,15 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  break;
 	case (T_DATA):
 	  /* current data */
-	  current_node->data = strdup(tok);
-	  logprintf("xmlparser: info: node data : %s\n", current_node->data);
+	  if (current_node->data) {
+	    /* avoid a memory leak */
+	    free(current_node->data);
+	  }
+	  current_node->data = lexer_decode_entities(tok);
+	  logprintf("info: node data : %s\n", current_node->data);
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -186,7 +218,7 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
       case STATE_NODE:
 	switch (res) {
 	case (T_IDENT):
-	  propertys = NULL;
+	  properties = NULL;
 	  current_property = NULL;
 
 	  /* save node name */
@@ -195,10 +227,10 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  }
 	  strcpy(node_name, tok);
 	  state = 2;
-	  logprintf("xmlparser: info: current node name \"%s\"\n", node_name);
+	  logprintf("info: current node name \"%s\"\n", node_name);
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -217,8 +249,8 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  subtree->name = strdup(node_name);
 
 	  /* set node propertys */
-	  subtree->props = propertys;
-	  logprintf("xmlparser: info: rec %d new subtree %s\n", rec, node_name);
+	  subtree->props = properties;
+	  logprintf("info: rec %d new subtree %s\n", rec, node_name);
 	  parse_res = xml_parser_get_node(subtree, node_name, rec + 1);
 	  if (parse_res != 0) {
 	    return parse_res;
@@ -241,9 +273,10 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  subtree->name = strdup (node_name);
 
 	  /* set node propertys */
-	  subtree->props = propertys;
+	  subtree->props = properties;
+
+	  logprintf("info: rec %d new subtree %s\n", rec, node_name);
 
-	  logprintf("xmlparser: info: rec %d new subtree %s\n", rec, node_name);
 	  if (current_subtree == NULL) {
 	    current_node->child = subtree;
 	    current_subtree = subtree;
@@ -260,11 +293,10 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  }
 	  strcpy(property_name, tok);
 	  state = 5;
-	  logprintf ("xmlparser: info: current property name \"%s\"\n",
-		     property_name);
+	  logprintf("info: current property name \"%s\"\n", property_name);
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -280,12 +312,12 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  if (strcmp(tok, root_name) == 0) {
 	    state = 4;
 	  } else {
-	    printf("xmlparser: error: xml struct, tok=%s, waited_tok=%s\n", tok, root_name);
+	    logprintf("error: xml struct, tok=%s, waited_tok=%s\n", tok, root_name);
 	    return -1;
 	  }
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -298,7 +330,7 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	  return 0;
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -321,20 +353,19 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	case (T_M_STOP_1):
 	  /* add a new property without value */
 	  if (current_property == NULL) {
-	    propertys = new_xml_property();
-	    current_property = propertys;
+	    properties = new_xml_property();
+	    current_property = properties;
 	  } else {
 	    current_property->next = new_xml_property();
 	    current_property = current_property->next;
 	  }
 	  current_property->name = strdup (property_name);
-	  logprintf ("xmlparser: info: new property %s\n",
-		     current_property->name);
+	  logprintf("info: new property %s\n", current_property->name);
 	  bypass_get_token = 1; /* jump to state 2 without get a new token */
 	  state = 2;
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -351,20 +382,19 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 	case (T_IDENT):
 	  /* add a new property */
 	  if (current_property == NULL) {
-	    propertys = new_xml_property();
-	    current_property = propertys;
+	    properties = new_xml_property();
+	    current_property = properties;
 	  } else {
 	    current_property->next = new_xml_property();
 	    current_property = current_property->next;
 	  }
 	  current_property->name = strdup(property_name);
-	  current_property->value = strdup(tok);
-	  logprintf ("xmlparser: info: new property %s=%s\n",
-		     current_property->name, current_property->value);
+	  current_property->value = lexer_decode_entities(tok);
+	  logprintf("info: new property %s=%s\n", current_property->name, current_property->value);
 	  state = 2;
 	  break;
 	default:
-	  printf("xmlparser: error: unexpected token \"%s\", state %d\n", tok, state);
+	  logprintf("error: unexpected token \"%s\", state %d\n", tok, state);
 	  return -1;
 	  break;
 	}
@@ -408,16 +438,16 @@ int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
 
 
       default:
-	printf("xmlparser: error: unknown parser state, state=%d\n", state);
+	logprintf("error: unknown parser state, state=%d\n", state);
 	return -1;
       }
     }
     /* lex error */
-    printf("xmlparser: error: lexer error\n");
+    logprintf("error: lexer error\n");
     return -1;
   } else {
     /* max recursion */
-    printf("xmlparser: error: max recursion\n");
+    logprintf("error: max recursion\n");
     return -1;
   }
 }
@@ -426,20 +456,17 @@ int xml_parser_build_tree(xml_node_t **root_node) {
   xml_node_t *tmp_node;
   int res;
 
-  *root_node = new_xml_node();
   tmp_node = new_xml_node();
   res = xml_parser_get_node(tmp_node, "", 0);
   if ((tmp_node->child) && (!tmp_node->child->next)) {
-    (*root_node)->name  = tmp_node->child->name;
-    (*root_node)->data  = tmp_node->child->data;
-    (*root_node)->props = tmp_node->child->props;
-    (*root_node)->child = tmp_node->child->child;
-    (*root_node)->next  = tmp_node->child->next;
+    *root_node = tmp_node->child;
+    free_xml_node(tmp_node);
+    res = 0;
   } else {
-    printf("xmlparser: error: xml struct\n");
+    logprintf("error: xml struct\n");
+    xml_parser_free_tree(tmp_node);
     res = -1;
   }
-  free(tmp_node);
   return res;
 }
 
@@ -450,10 +477,10 @@ char *xml_parser_get_property (xml_node_t *node, const char *name) {
   prop = node->props;
   while (prop) {
 
-    logprintf ("xmlparser: looking for %s in %s\n", name, prop->name);
+    logprintf ("looking for %s in %s\n", name, prop->name);
 
     if (!strcasecmp (prop->name, name)) {
-      logprintf ("xmlparser: found it. value=%s\n", prop->value);
+      logprintf ("found it. value=%s\n", prop->value);
       return prop->value;
     }
 
@@ -496,6 +523,40 @@ int xml_parser_get_property_bool (xml_node_t *node, const char *name,
     return 0;
 }
 
+static int xml_escape_string_internal (char *buf, const char *s,
+				       xml_escape_quote_t quote_type)
+{
+  int c, length = 0;
+  int sl = buf ? 8 : 0;
+  /* calculate max required buffer size */
+  while ((c = *s++ & 0xFF))
+    switch (c)
+    {
+    case '"':  if (quote_type != XML_ESCAPE_DOUBLE_QUOTE) goto literal;
+	       length += snprintf (buf + length, sl, "&quot;"); break;
+    case '\'': if (quote_type != XML_ESCAPE_SINGLE_QUOTE) goto literal;
+	       length += snprintf (buf + length, sl, "&apos;"); break;
+    case '&':  length += snprintf (buf + length, sl, "&amp;");  break;
+    case '<':  length += snprintf (buf + length, sl, "&lt;");   break;
+    case '>':  length += snprintf (buf + length, sl, "&gt;");   break;
+    case 127:  length += snprintf (buf + length, sl, "&#127;"); break;
+    case '\t':
+    case '\n':
+      literal: if (buf)	buf[length] = c; ++length; break;
+    default:   if (c >= ' ') goto literal;
+	       length += snprintf (buf + length, sl, "&#%d;", c); break;
+    }
+  if (buf)
+    buf[length] = 0;
+  return length + 1;
+}
+
+char *xml_escape_string (const char *s, xml_escape_quote_t quote_type)
+{
+  char *buf = malloc (xml_escape_string_internal (NULL, s, quote_type));
+  return buf ? (xml_escape_string_internal (buf, s, quote_type), buf) : NULL;
+}
+
 static void printf_indent (int indent, const char *format, ...) {
 
   int     i ;
@@ -523,7 +584,9 @@ static void xml_parser_dump_node (xml_node_t *node, int indent) {
 
   p = node->props;
   while (p) {
-    printf ("%s='%s'", p->name, p->value);
+    char *value = xml_escape_string (p->value, XML_ESCAPE_SINGLE_QUOTE);
+    printf ("%s='%s'", p->name, value);
+    free (value);
     p = p->next;
     if (p) {
       printf ("\n");
@@ -546,4 +609,3 @@ static void xml_parser_dump_node (xml_node_t *node, int indent) {
 void xml_parser_dump_tree (xml_node_t *node) {
   xml_parser_dump_node (node, 0);
 }
-
